import datetime
from unittest.mock import patch

from risc_generator import Protections
from risc_generator.domain.contract import FPQ1Contract
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractTest(ContractTestBase):
    def setUp(self) -> None:
        self.a_protections = Protections(protections=self.a_dict_of_protections)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAFRContractWithSomeDate_whenFormattedTextDateFrLocale_thenReturnFrDate(self, faker_mock):
        a_year = 2022
        a_month = 1
        a_day = 2

        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.formatted_text_date()
        expected = "Le 2 janvier 2022"
        self.assertEqual(expected, actual)

    @patch("risc_generator.domain.contract.Faker")
    def test_givenAContractWithSomeDate_whenFormattedTextDateEnLocale_thenReturnEnDate(self, faker_mock):
        a_year = 2022
        a_month = 1
        a_day = 2

        faker_mock().date_between.return_value = datetime.date(year=a_year, month=a_month, day=a_day)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.formatted_text_date()
        expected = f"January 2, {a_year}"
        self.assertEqual(expected, actual)

    def test_givenAContractWithSomeProtections_whenSTR_thenReturnProperString(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_locale,
        )

        actual = str(fpq1_contract)

        expected = f"FPQ1 contract with the following protections:\n{self.a_protections}"

        self.assertEqual(expected, actual)

    def test_givenAContratWithFAQ5A_whenCarOwnership_thenReturnRentalInLocaleLang(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ5ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)

        # FR locale

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Location"

        self.assertEqual(expected, actual)

        # EN locale
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Rental"

        self.assertEqual(expected, actual)

    def test_givenAContratWithFAQ23A_whenCarOwnership_thenReturnPurchaseInLocaleLang(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ23ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)

        # FR locale

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Achat"

        self.assertEqual(expected, actual)

        # EN locale
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Purchase"

        self.assertEqual(expected, actual)

    def test_givenAContratWithoutFAQ23AOrFAQ5A_whenCarOwnership_thenReturnPurchaseInLocaleLang(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_protections = Protections(protections=a_dict_of_protections)

        # FR locale

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Achat"

        self.assertEqual(expected, actual)

        # EN locale
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.car_ownership()

        expected = "Purchase"

        self.assertEqual(expected, actual)

    def test_givenAContratWithFAQ5A_whenCarWitFinancing_thenReturnLessor(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ5ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_with_financing()

        expected = "lessor"

        self.assertEqual(expected, actual)

    def test_givenAContratWithFAQ23A_whenCarWitFinancing_thenReturnCreditor(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ23ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_with_financing()

        expected = "creditor"

        self.assertEqual(expected, actual)

    def test_givenAContratWithoutFAQ5AOrFAQ23A_whenCarWitFinancing_thenReturnNone(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_protections = Protections(protections=a_dict_of_protections)

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.car_with_financing()

        expected = None

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq34ab_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 34 (A-B)\n"
        "Accident benefits insurance\n"
        "(change to amount of insurance or insured persons)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement changes Q.E.F. N 34 entitled ''Accident benefits'' of the insurance "
        "contract, based on the applicable option.\n"
        "To determine which option applies, see Item 4, ''Declarations'' of the insurance contract.\n"
        "Option 34A – Changes when the named insured is an individual\n"
        "This option provides that, for the persons named in the table below:\n"
        "\t the maximum amounts shown in the table will change those shown in Q.E.F. N "
        "34 by replacing or, where applicable, adding to them;\n"
        "\t coverage will apply only for the divisions and subdivisions where an amount, "
        "an insurance premium or a rebate is shown in the table below, or entered specifically for "
        "this endorsement in the ''Declarations'' section of the insurance contract:\n"
        "MAXIMUM AMOUNTS\n"
        "DIVISION 1 | DIVISION 2\n"
        "Name of insured person | Date of birth | Subdivision A Death benefits (principal sum) | "
        "Subdivision B Dismemberment benefits (principal sum) | "
        "Subdivision C Reimbursement of medical expenses (maximum) | DIVISION 2 Total "
        "disability benefits (maximum, per week) | Insurance premium or rebate\n"
        "1. |  | $ | $ | $ | $ | $\n"
        "2. |  | $ | $ | $ | $ | $\n"
        "3. |  | $ | $ | $ | $ | $\n"
        "4. |  | $ | $ | $ | $ | $\n"
        " |  |  |  |  | Total: |  $\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Option 34B – Changes when the named insured is a legal person, partnership or association\n"
        "When the named insured is a legal person, partnership or association, Q.E.F. N 34 will be changed "
        "as follows:\n"
        "\t only the employees, shareholders, members or partners named in the table below will "
        "be considered ''named insureds'';\n"
        "\t coverage will apply only for the divisions and subdivisions where an amount, "
        "an insurance premium or a rebate is shown in the table below, or entered specifically "
        "for this endorsement in the ''Declarations'' section of the insurance contract:\n"
        "MAXIMUM AMOUNTS\n"
        "DIVISION 1 | DIVISION 2\n"
        "Name of insured person | Date of birth | Subdivision A Death benefits (principal sum) | "
        "Subdivision B Dismemberment benefits (principal sum) | "
        "Subdivision C Reimbursement of medical expenses (maximum) | DIVISION 2 Total "
        "disability benefits (maximum, per week) | Insurance premium or rebate\n"
        "1. |  | $ | $ | $ | $ | $\n"
        "2. |  | $ | $ | $ | $ | $\n"
        "3. |  | $ | $ | $ | $ | $\n"
        "4. |  | $ | $ | $ | $ | $\n"
        " |  |  |  |  | Total: |  $\n"
        "All other conditions of the insurance contract remain the same."
    )

    return text



from risc_generator import Protections, FPQ1Contract
from tests.domain.contract.contract_base import ContractTestBase


class ContractCancellationOfPolicyTest(ContractTestBase):
    def test_givenAFRContract_whenGenerateCancellationOfPolicyText_thenReturnProperFRText(self):
        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fpq1_contract.generate_cancellation_of_policy_text()

        a_general_phone_number = "123 456-7890"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC\n"
            "F.P.Q. N 1 - FORMULAIRE DES PROPRIÉTAIRES\n"
            f"Numéro de police : {a_policy_number}\n\n"
            "RÉSILIATION DE LA POLICE\n"
            "Ne pas utiliser cette formule si votre police couvre plusieurs véhicules et qu'un seul doit être résilié. "
            "Veuillez communiquer avec le service à la clientèle au numéro inscrit ci-dessous.\n"
            f"AVANT DE RÉSILIER communiquez avec le service à la clientèle au {a_general_phone_number} ou "
            f"{a_general_phone_number_1_800}; ceci pourrait vous éviter de perdre certains avantages.\n"
            f"Je demande la résiliation complète de la police numéro {a_policy_number}, de ses avenants, de ses "
            f"renouvellements et s'il y a lieu, le remboursement de la prime non acquise à compter du : "
            f"______________      ______________      ______________\n"
            f"(Année) (Mois) (Jour)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n"
            f"(Assuré) (Année) (Mois) (Jour)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n"
            f"(Assuré additionnel) (Année) (Mois) (Jour)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n "
            f"(Assuré additionnel) (Année) (Mois) (Jour)\n"
            f"Raison de l'annulation :  _____________________________________________  "
            f"Nouvel assureur :  ________________________________________\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateCancellationOfPolicyText_thenReturnProperENText(self):
        a_protections = Protections(protections=self.a_dict_of_protections.copy())

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=self.a_premium_mock,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = fpq1_contract.generate_cancellation_of_policy_text()

        a_general_phone_number = "123 456-7890"
        a_general_phone_number_1_800 = "1 800 123-4567"
        a_policy_number = "1234567-001"
        a_new_page = "<###NEW_PAGE###>"

        expected = (
            "QUEBEC AUTOMOBILE INSURANCE POLICY\n"
            "Q.P.F. B 1 - OWNER'S FORM\n"
            f"Policy Number: {a_policy_number}\n\n"
            f"CANCELLATION OF POLICY\n"
            f"If your policy covers several vehicles and coverage for only one of them must be cancelled, "
            f"do not use this form.\n"
            f"Please contact Customer Service at the number listed below.\n"
            f"BEFORE CANCELLING contact Customer Service at {a_general_phone_number} or "
            f"{a_general_phone_number_1_800}, this could avoid you losing some of your advantages.\n"
            f"I hereby request the full cancellation of policy number {a_policy_number}, "
            f"its endorsements, its renewals and if applicable, the reimbursement of any unearned premium effective: "
            f"______________      ______________      ______________\n"
            f"(Year) (Month) (Day)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n"
            f"(Insured) (Year) (Month) (Day)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n"
            f"(Additional Insured) (Year) (Month) (Day)\n"
            f"Signature :  ____________________________________________________________   "
            f"Date : _____________     _____________     _____________\n "
            f"(Additional Insured) (Year) (Month) (Day)\n"
            f"Reason of cancellation:  _____________________________________________  "
            f"New insurer:  ________________________________________\n"
            f"{a_new_page}\n"
        )

        self.assertEqual(expected, actual)



from risc_generator import FPQ1Contract, Protections, Premium
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase


class ArticlesHeaderTest(ContractTestBase):
    def setUp(self) -> None:
        self.a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        self.a_premium = Premium(**protection_premiums)

    def test_givenAFRContract_whenGenerateHeaderText_thenReturnProperFRText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        a_policy_number = "1234567-001"

        expected = (
            "POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC\n"
            "F.P.Q. N 1 - FORMULAIRE DES PROPRIÉTAIRES\n"
            f"Numéro de police : {a_policy_number}\n\n"
            "CONDITIONS PARTICULIÈRES\n"
            "ÉMISSION de votre nouvelle police d'assurance\n"
        )

        actual = fr.declaration_articles.generate_declaration_header_text(fpq1_contract.policy_number)

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateHeaderText_thenReturnProperENText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        a_policy_number = "1234567-001"
        expected = (
            "QUEBEC AUTOMOBILE INSURANCE POLICY\n"
            "Q.P.F. B 1 - OWNER'S FORM\n"
            f"Policy Number: {a_policy_number}\n\n"
            "DECLARATIONS\n"
            "ISSUE of your new insurance policy\n"
        )

        actual = en.declaration_articles.generate_declaration_header_text(fpq1_contract.policy_number)

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq30_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 30\n"
        "Limitation of coverage for equipment and machinery attached to vehicle\n"
        "(Section A)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "Subject to the Automobile Insurance Act, this endorsement limits coverage "
        "under Section A of the insurance contract by adding the following exclusion:\n"
        "\t Coverage will not be provided for the financial consequences that an insured "
        "person may incur if held civilly liable for damage caused by the following "
        "equipment or machinery, or accessories thereof, when mounted on or attached to the specified vehicle:\n"
        " \n"
        "(description of equipment or machinery)\n"
        "For the above exclusion to apply:\n"
        "\t damage must be caused while the equipment or machinery, or accessories thereof, "
        "are at the site of their use; and\n"
        "\t the insured person's civil liability must result from his or her owning the "
        "equipment or machinery, or accessories thereof, or from their use or their operation.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..config import (
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    NEW_PAGE_TAG,
    FAKE_ROADSIDE_ASSISTANCE_PHONE_NUMBER,
    FAKE_ROADSIDE_PROGRAM_NAME,
    FAKE_FIDELITY_PROGRAM_NAME_EN,
)


def generate_important_instructions_text(self) -> str:
    important_instructions = (
        "IMPORTANT INSTRUCTIONS\n\n"
        f"{self.formatted_text_date()}\n"
        "Please find hereafter instructions on very important parts of your insurance policy; we suggest that you "
        "read them carefully and take any required action, if necessary.\n\n"
    )

    fidelity = (
        f"{FAKE_FIDELITY_PROGRAM_NAME_EN} Program\n\n"
        "Because we appreciate your loyalty, we're pleased to offer you privileges that accumulate with your years "
        f"of loyalty, thanks to our {FAKE_FIDELITY_PROGRAM_NAME_EN} Program. Here are the details regarding your "
        "privileges, which you can also find in your Client Centre.\n"
        "Your recognized loyalty: New client\n"
        "Your privileges:\n"
        "\t Exclusive partner offers\n"
        "\t Legal assistance\n"
        "\t Psychological assistance service\n"
    )

    if self.protections.is_protected("FAQ41Coverage"):
        fidelity += "\t No deductible to pay in the case of a total loss or hit-and-run (auto)\n"

    fidelity += (
        "This list contains all the privileges associated with your recognized loyalty. Please note that the "
        "new privileges will be available at the renewal of each insurance policy held by a member of your "
        "household. Please see the Declarations or Coverage Summary Page(s) of your various insurance contracts held "
        "with us to learn more about applicable coverage details.\n"
    )

    if (
        self.protections.is_protected("FAQ33Coverage")
        or self.protections.is_protected("FAQ5ACoverage")
        or self.protections.is_protected("FAQ23ACoverage")
    ):
        notice_to_the_insuree = "Notice to the insured\n"
    else:
        notice_to_the_insuree = ""

    if self.protections.is_protected("FAQ33Coverage"):
        notice_to_the_insuree += (
            f"AUTOMOBILE POLICY No {self.policy_number}\n"
            f"You have chosen {FAKE_ROADSIDE_PROGRAM_NAME} Roadside Assistance, which provides you with 24-hour "
            "roadside assistance throughout Canada and the United States, 365 days a year.\n"
            "Therefore, three weeks after the effective date of the Q.E.F. endorsement N 33 Roadside Assistance "
            "Coverage, you will receive a kit for each eligible vehicle, including your membership card and your "
            "user's guide.\n"
            "As soon as the endorsement Q.E.F. N 33 comes into effect, you will be able to benefit from "
            f"this service even if you have not received your {FAKE_ROADSIDE_PROGRAM_NAME} roadside assistance kit."
            f"In case of need, simply call {FAKE_ROADSIDE_ASSISTANCE_PHONE_NUMBER} (24 hours a day, 7 days a week) "
            "and mention the policy number of the covered vehicle to the agent.\n"
        )

    if self.protections.is_protected("FAQ5ACoverage") or self.protections.is_protected("FAQ23ACoverage"):
        if self.protections.is_protected("FAQ5ACoverage"):
            creditor_type = "locator"
        else:
            creditor_type = "creditor"
        notice_to_the_insuree += (
            f"AUTOMOBILE POLICY No {self.policy_number} - Vehicle 1 : "
            f"{self.vehicle.format_vehicle_make_model_year_details()}\n"
            f"We are enclosing a copy of your policy to give to your {creditor_type}, "
            "if requested. It is included at the very end of this mailing.\n"
        )

    actions_to_take = (
        "\nActions Required\n\n"
        "Automobile insurance\n"
        "\t Detach and keep your insurance certificates with you.\n"
        "\t Detach and keep the automobile rental certificate with your driver's licence. This way, you will avoid "
        "paying additional insurance fees to the car rental agency.\n"
        "\t Please read the Coverage Summary section of your Quebec automobile insurance contract to ensure that "
        "your coverages are well-suited to your needs.\n"
        f"\t Don't hesitate to contact customer service at {GENERAL_FAKE_PHONE_NUMBER_1_800} for any questions "
        "regarding your contract.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    text = important_instructions + fidelity + notice_to_the_insuree + NEW_PAGE_TAG + "\n" + actions_to_take

    return text



import random

from faker import Faker


class PaymentMethodFaker(Faker):
    def __init__(self, locale: str) -> None:
        """
        locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        """
        if locale == "fr_CA":
            self.bill = "facture"
            self.pre_authorize = "paiement"
        elif locale == "en_CA":
            self.bill = "bill"
            self.pre_authorize = "pre-authorize"
        super().__init__(locale)

    def payment_method(self) -> str:
        """
        A faker function to fake a payment method.

        Return:
            The payment method, either 'bill' or 'pre-authorize'.
        """
        return random.choices([self.bill, self.pre_authorize], weights=[0.05, 0.95], k=1)[0]



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq34ab_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 34 (A-B)\n"
        "Assurance de personnes\n"
        "(modifications des montants d'assurance ou des personnes assurées)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant modifie le F.A.Q. N 34 intitulé « Assurance de personnes » du contrat d'assurance, "
        "selon l'option applicable.\n"
        "Pour connaître l‘option applicable, voir l'article 4 de la section « Conditions "
        "particulières » du contrat d'assurance.\n"
        "Option 34A – Modifications lorsque l'assuré désigné est un individu\n"
        "Cette option prévoit que pour les personnes désignées au tableau ci-dessous :\n"
        "\t les montants maximums écrits au tableau modifient ceux du F.A.Q. N 34 en les remplaçant "
        "ou, si le cas se présente, en s'ajoutant;\n"
        "seules sont applicables les divisions et les subdivisions pour lesquelles un montant, "
        "une prime d'assurance ou une ristourne est écrit au tableau, ou écrit spécifiquement pour "
        "cet avenant à la section « Conditions particulières » du contrat d'assurance.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "MONTANTS MAXIMUMS\n"
        "DIVISION 1 | DIVISION 2\n"
        "Nom de la personne assurée | Date de naissance | Subdivision A Indemnités en cas de "
        "décès (capital assuré) | Subdivision B Indemnités en cas de mutilation (capital assuré) | "
        "Subdivision C Remboursement de frais médicaux (montant maximum) | DIVISION 2 Indemnités "
        "en cas d'incapacité totale (montant maximum, par semaine) | Primes d'assurance ou ristournes\n"
        "1. |  | $ | $ | $ | $ | $\n"
        "2. |  | $ | $ | $ | $ | $\n"
        "3. |  | $ | $ | $ | $ | $\n"
        "4. |  | $ | $ | $ | $ | $\n"
        " |  |  |  |  | Total: |  $\n"
        "Option 34B – Modifications lorsque l'assuré désigné est une personne morale, une "
        "société ou une association"
        "Lorsque l'assuré désigné est une personne morale, une société ou une association, le F.A.Q. "
        "N 34 est modifié de la façon suivante :\n"
        "\t seuls les employés, actionnaires, membres ou associés désignés au tableau ci-dessous "
        "sont considérés comme des « assurés désignés »;\n"
        "\t seules sont applicables les divisions et les subdivisions pour lesquelles un montant, "
        "une prime d'assurance ou une ristourne est écrit au tableau ci-dessous, ou écrit spécifiquement "
        "pour cet avenant à la section « Conditions particulières » du contrat d'assurance :\n"
        "MONTANTS MAXIMUMS\n"
        "DIVISION 1 | DIVISION 2\n"
        "Nom de la personne assurée | Date de naissance | Subdivision A Indemnités en cas de "
        "décès (capital assuré) | Subdivision B Indemnités en cas de mutilation (capital assuré) | "
        "Subdivision C Remboursement de frais médicaux (montant maximum) | DIVISION 2 Indemnités "
        "en cas d'incapacité totale (montant maximum, par semaine) | Primes d'assurance ou ristournes\n"
        "1. |  | $ | $ | $ | $ | $\n"
        "2. |  | $ | $ | $ | $ | $\n"
        "3. |  | $ | $ | $ | $ | $\n"
        "4. |  | $ | $ | $ | $ | $\n"
        " |  |  |  |  | Total: |  $\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from .bill import *
from .cancellation_of_policy import *
from .certificates import *
from .client_advantages import *
from .declaration import *
from .declaration_articles import *
from .endorsements import *
from .fpq_1 import *
from .general_header import *
from .header import *
from .important_instructions import *



from typing import Dict, Union

from .tools import change_to_snake_case


class Premium:
    def __init__(self, **kwargs: Dict) -> None:
        """
        A premium for the protections that generate a premium increase. Namely, liability coverage, property damage
        coverage (B1 to B4), and endorsement 20A, 33, 34 and 43.

        **kwargs (dict): A dictionary where the keys are the protections included in the contract coverage and the
        values are the premium amount.
        """
        for key, value in kwargs.items():
            # We save attribute in snake case format and use the same function in a getter method
            # to get the proper attribute using the CamelCase coverage format.
            self.__setattr__(change_to_snake_case(key), value)

        self.total = sum(list(self.__dict__.values()))  # The total premium amount

    def get_premium(self, item: str) -> Union[str, None]:
        item = change_to_snake_case(item)
        value = None
        if self.get(item) is not None:
            value = str(self.get(item))
        return value

    def get(self, item: str) -> Union[int, None]:
        """
        Method to get a premium value for a specific protection (item). If the premium does not include the
        protection premium, it means it is not covered. Thus, we return a None.

        item (str): Item (the protection) we want the premium amount.

        Return:
            A int of the premium amount or a None if not covered.
        """
        try:
            item = change_to_snake_case(item)
            return getattr(self, item)
        except AttributeError:
            return None



import random

from faker import Faker

from .vehicle_provider import VehicleProvider
from ...domain.vehicle import Vehicle
from ...faker.vehicule_faker.purchase_condition_faker import (
    QuebecPurchaseConditionFaker,
)
from ...faker.vehicule_faker.tools import is_valid_vehicle_year, generate_vin


class QuebecVehicleFaker(Faker):
    def __init__(self, locale: str) -> None:
        """
        locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        """
        super().__init__(locale=locale)
        self.vehicle_faker = Faker(locale=locale)
        self.vehicle_faker.add_provider(VehicleProvider)
        self.purchase_condition_faker = QuebecPurchaseConditionFaker(locale=locale)

    def vehicle(self, faq_43: bool, faq_5a: bool, contract_year: int) -> Vehicle:
        """
        Method to fake a vehicle.

        faq_43 (bool): Either or not the car need to respect FAQ43 endorsement.
        contract_year (bool): Year of the contract.

        Return:
            A faked Vehicle.
        """
        if faq_43:
            # If there is a FAQ 43, the vehicle need to be 5 years old or less

            invalid = True
            while invalid:  # We loop until the vehicle year is valid
                vehicle = self.vehicle_faker.vehicle_object()

                vehicle_year = vehicle.get("Year")
                if is_valid_vehicle_year(vehicle_year=vehicle_year, contract_year=contract_year):
                    # A vehicle year is valid when the difference between
                    # the starting year of the contrat and the age of the vehicle is
                    # smaller than 5. For example, given a starting year of the contract
                    # in 2020, vehicle needs to be at most a 2015 model.
                    invalid = False
                    vehicle_details = vehicle
        else:
            vehicle_details = self.vehicle_faker.vehicle_object()

        vehicle_details.update({"vin": generate_vin()})

        if faq_5a:
            # Endorsement FAQ5A is for car in rental, thus the car is necessarily new.
            vehicle_details.update({"purchase_condition": self.purchase_condition_faker.new_purchase_condition()})
        else:
            vehicle_details.update({"purchase_condition": self.purchase_condition_faker.purchase_condition()})

        # We lowercase the key since some are capitalize
        vehicle_details = {key.lower(): value for key, value in vehicle_details.items()}

        return Vehicle(**vehicle_details)

    def seed_instance(self, seed=42) -> None:
        """
        seed (int): The seed value to use for the generation seed. By default, 42.
        """
        super().seed_instance(seed)
        self.vehicle_faker.seed_instance(seed)
        self.purchase_condition_faker.seed_instance(seed)
        random.seed(seed)
